@@ -19,6 +19,7 @@ from pamqp import frame
 from pamqp import exceptions as pamqp_exceptions
 from pamqp import specification
 
+from rabbitpy import DEBUG
 from rabbitpy import base
 from rabbitpy import events
 from rabbitpy import exceptions
@@ -54,10 +55,12 @@ class IOLoop(object):
 
     def run(self):
         self._data.running = True
-        while (self._data.running and
-               not self._data.events.is_set(events.SOCKET_CLOSE)):
+        while self._data.running:
             try:
                 self._poll()
+            except (KeyboardInterrupt, SystemExit) as exception:
+                LOGGER.warning('Exiting IOLoop.run due to %s', exception)
+                break
             except EnvironmentError as exception:
                 if (getattr(exception, 'errno', None) == errno.EINTR or
                     (isinstance(getattr(exception, 'args', None), tuple) and
@@ -65,11 +68,28 @@ class IOLoop(object):
                      exception.args[0] == errno.EINTR)):
                     continue
 
+            if self._data.events.is_set(events.SOCKET_CLOSE):
+                if DEBUG:
+                    LOGGER.debug('Exiting due to closed socket')
+                break
+        if DEBUG:
+            LOGGER.debug('Exiting IOLoop.run')
+
     def stop(self):
+        LOGGER.info('Stopping IOLoop')
         self._data.running = False
+        try:
+            self._data.write_trigger.close()
+        except socket.error:
+            pass
 
     def _poll(self):
         # Poll select with the materialized lists
+        #if DEBUG:
+        #    LOGGER.debug('Polling')
+        if not self._data.running:
+            LOGGER.info('Exiting poll')
+
         if self._data.write_queue.empty() and not self._data.failed_write:
             read, write, err = select.select(*self._data.read_only)
         else:
@@ -78,6 +98,7 @@ class IOLoop(object):
             self._data.running = False
             self._data.error_callback(None)
             return
+
         # Clear out the trigger socket
         if self._data.write_trigger in read:
             self._data.write_trigger.recv(1024)
@@ -86,8 +107,14 @@ class IOLoop(object):
             self._read()
         if write:
             self._write()
+        #if DEBUG:
+        #    LOGGER.debug('End of poll')
 
     def _read(self):
+        if not self._data.running:
+            if DEBUG:
+                LOGGER.debug('Skipping read, not running')
+            return
         try:
             if self._data.ssl:
                 self._data.read_callback(self._data.fd.read(MAX_READ))
@@ -118,6 +145,10 @@ class IOLoop(object):
                 break
 
     def _write_frame(self, channel, value):
+        if not self._data.running:
+            if DEBUG:
+                LOGGER.debug('Skipping write frame, not running')
+            return
         frame_data = frame.marshal(value, channel)
         try:
             self._data.fd.sendall(frame_data)
@@ -147,6 +178,8 @@ class IO(threading.Thread, base.StatefulObject):
         # A socket to trigger write interrupts with
         (self._write_listener,
          self._write_trigger) = socket.socketpair()
+        self._write_listener.setblocking(0)
+        self._write_trigger.setblocking(0)
 
         self._buffer = bytes()
         self._channels = dict()
@@ -187,7 +220,7 @@ class IO(threading.Thread, base.StatefulObject):
         del self._socket
         del self._channels
         del self._buffer
-        LOGGER.debug('Exiting IO')
+        LOGGER.info('Exiting IO.run')
 
     def on_error(self, exception):
         """Common functions when a socket error occurs. Make sure to set closed
@@ -197,27 +230,44 @@ class IO(threading.Thread, base.StatefulObject):
 
         """
         LOGGER.error('Socket error: %s', exception, exc_info=True)
+        args = [self._args['host'], self._args['port'], exception[1]]
+        if self._channels[0][0].open:
+            self._exceptions.put(exceptions.ConnectionResetException(*args))
+        else:
+            self._exceptions.put(exceptions.ConnectionException(*args))
         self._set_state(self.CLOSED)
-        self._exceptions.put(exception)
+        self._loop.stop()
+        try:
+            self._write_trigger.send('0')
+        except socket.error:
+            pass
         self._events.clear(events.SOCKET_OPENED)
         self._events.set(events.SOCKET_CLOSED)
         self._events.set(events.EXCEPTION_RAISED)
-        if self._events.is_set(events.CHANNEL0_OPENED):
-            self._events.set(events.CHANNEL0_CLOSE)
-            self._events.wait(events.CHANNEL0_CLOSED)
         if self.open:
             self._close()
-        self._events.set(events.EXCEPTION_RAISED)
-        raise exception
+        self._channels[0][0]._set_state(self._channels[0][0].CLOSED)
 
     def on_read(self, data):
         # Append the data to the buffer
         self._buffer += data
 
-        # Read and process data
-        value = self._read_frame()
-        if value[0] is not None:
-            LOGGER.debug('Received (%i) %r', value[0], value[1])
+        while self._buffer:
+
+            # Read and process data
+            value = self._read_frame()
+
+            # Break out if a frame could not be decoded
+            if self._buffer and value[0] is None:
+                break
+
+            if DEBUG:
+                LOGGER.debug('Received (%i) %r', value[0], value[1])
+
+            # If it's channel 0, call the Channel0 directly
+            if value[0] == 0:
+                self._channels[0][0].on_frame(value[1])
+                continue
 
             # Close the channel if it's a Channel.Close frame
             if isinstance(value[1], specification.Channel.Close):
@@ -243,7 +293,9 @@ class IO(threading.Thread, base.StatefulObject):
         :param pamqp.specification.Frame frame_value: The frame to add
 
         """
-        #LOGGER.debug('Adding %s to channel %s', frame_value.name, channel_id)
+        if DEBUG:
+            LOGGER.debug('Adding %s to channel %s',
+                         frame_value.name, channel_id)
         self._channels[channel_id][1].put(frame_value)
 
     def _close(self):
@@ -255,7 +307,8 @@ class IO(threading.Thread, base.StatefulObject):
 
     def _connect_socket(self, sock, address):
         """Connect the socket to the specified host and port."""
-        LOGGER.debug('Connecting to %r', address)
+        if DEBUG:
+            LOGGER.debug('Connecting to %r', address)
         sock.settimeout(self.CONNECTION_TIMEOUT)
         sock.connect(address)
 
@@ -273,13 +326,17 @@ class IO(threading.Thread, base.StatefulObject):
                 self._connect_socket(sock, sockaddr)
                 break
             except socket.error as error:
-                LOGGER.debug('Error connecting to %r: %s', sockaddr, error)
+                if DEBUG:
+                    LOGGER.debug('Error connecting to %r: %s', sockaddr, error)
                 sock = None
                 continue
 
         if not sock:
-            raise exceptions.ConnectionException(self._args['host'],
-                                                 self._args['port'])
+            args = [self._args['host'], self._args['port'], 'Could not connect']
+            self._exceptions.put(exceptions.ConnectionException(*args))
+            self._events.set(events.EXCEPTION_RAISED)
+            return
+
         self._socket = sock
         self._socket.settimeout(0)
         self._events.set(events.SOCKET_OPENED)
@@ -293,7 +350,8 @@ class IO(threading.Thread, base.StatefulObject):
         """
         sock = socket.socket(af, socktype, proto)
         if self._args['ssl']:
-            LOGGER.debug('Wrapping socket for SSL')
+            if DEBUG:
+                LOGGER.debug('Wrapping socket for SSL')
             return ssl.wrap_socket(sock,
                                    self._args['ssl_key'],
                                    self._args['ssl_cert'],
@@ -341,7 +399,8 @@ class IO(threading.Thread, base.StatefulObject):
             return value, None, None
         except specification.AMQPFrameError as error:
             LOGGER.error('Failed to demarshal: %r', error, exc_info=True)
-            LOGGER.debug(value)
+            if DEBUG:
+                LOGGER.debug(value)
             return value, None, None
         return value[byte_count:], channel_id, frame_in
 
@@ -380,7 +439,8 @@ class IO(threading.Thread, base.StatefulObject):
 
         """
         self._connect()
-        LOGGER.debug('Socket connected')
+        if DEBUG:
+            LOGGER.debug('Socket connected')
 
         # Create the remote name
         address, port = self._socket.getsockname()
@@ -388,6 +448,7 @@ class IO(threading.Thread, base.StatefulObject):
         self._remote_name = '%s:%s -> %s:%s' % (address, port,
                                                 peer_address, peer_port)
         self._loop = IOLoop(self._socket, self.on_error, self.on_read,
-                            self._write_queue, self._events,
+                            self._write_queue,
+                            self._events,
                             self._write_listener)
         self._loop.run()
